@@ -12,16 +12,17 @@ from benchmark_runner.common.virtctl.virtctl import Virtctl
 from benchmark_runner.common.template_operations.template_operations import TemplateOperations
 from benchmark_runner.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
 from benchmark_runner.common.ssh.ssh import SSH
-from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNonInstalled, EmptyLSOPath
+from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNotInstalled, CNVNotInstalled, KataNotInstalled, EmptyLSOPath
 from benchmark_runner.main.environment_variables import environment_variables
 from benchmark_runner.common.clouds.shared.s3.s3_operations import S3Operations
 from benchmark_runner.common.prometheus.prometheus_snapshot import PrometheusSnapshot
 from benchmark_runner.common.prometheus.prometheus_snapshot_exceptions import PrometheusSnapshotError
+from benchmark_runner.common.prometheus.prometheus_metrics_operations import PrometheusMetricsOperation
 
 
 class BenchmarkOperatorWorkloadsOperations:
     """
-    This class contains all the custom_workloads
+    This class contains benchmark-operator workload operations
     """
     def __init__(self):
         # environment variables
@@ -38,7 +39,6 @@ class BenchmarkOperatorWorkloadsOperations:
         self._elasticsearch = self._environment_variables_dict.get('elasticsearch', '')
         self._run_artifacts = self._environment_variables_dict.get('run_artifacts', '')
         self._run_artifacts_path = self._environment_variables_dict.get('run_artifacts_path', '')
-        self._lso_path = self._environment_variables_dict.get('lso_path', '')
         self._date_key = self._environment_variables_dict.get('date_key', '')
         self._key = self._environment_variables_dict.get('key', '')
         self._endpoint_url = self._environment_variables_dict.get('endpoint_url', '')
@@ -73,10 +73,22 @@ class BenchmarkOperatorWorkloadsOperations:
         # PrometheusSnapshot
         if self._enable_prometheus_snapshot:
             self._snapshot = PrometheusSnapshot(oc=self._oc, artifacts_path=self._run_artifacts_path, verbose=True)
+        self._prometheus_metrics_operation = PrometheusMetricsOperation()
+        # Extract lso id for LSO workload
+        if '_lso' in self._environment_variables_dict.get('workload'):
+            # Update lso_disk_id only if both worker_disk_ids and a free disk exist
+            if self._environment_variables_dict.get('worker_disk_ids', '') and self._oc.get_free_disk_id(
+                    node=self._environment_variables_dict['lso_node']):
+                self._lso_disk_id = self._oc.get_free_disk_id(node=self._environment_variables_dict['lso_node'])
+                self._environment_variables_dict['lso_disk_id'] = self._lso_disk_id
+            else:
+                self._lso_disk_id = self._environment_variables_dict.get('lso_disk_id', '')
+        else:
+            self._lso_disk_id = None
 
     def set_login(self, kubeadmin_password: str = ''):
         """
-        This method set oc login
+        This method sets oc login
         :param kubeadmin_password:
         :return:
         """
@@ -87,7 +99,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def update_node_selector(self, runner_path: str = environment_variables.environment_variables_dict['runner_path'], yaml_path: str = '', pin_node: str = ''):
         """
-        This method update node selector in yaml
+        This method updates node selector in yaml
         @return:
         """
         data = []
@@ -112,7 +124,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def make_deploy_benchmark_controller_manager(self, runner_path: str = environment_variables.environment_variables_dict['runner_path']):
         """
-        This method make deploy benchmark operator
+        This method deploys the benchmark operator
         :return:
         """
         benchmark_operator_path = 'benchmark-operator'
@@ -129,7 +141,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def make_undeploy_benchmark_controller_manager(self, runner_path: str = environment_variables.environment_variables_dict['runner_path']):
         """
-        This method make undeploy benchmark operator
+       This method removes a benchmark-operator deployment
         :return:
         """
         benchmark_operator_path = 'benchmark-operator'
@@ -142,7 +154,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def make_undeploy_benchmark_controller_manager_if_exist(self, runner_path: str = environment_variables.environment_variables_dict['runner_path']):
         """
-        This method make undeploy benchmark controller manager if exist
+        This method undeploys benchmark controller manager, if it exists
         @return:
         """
         # delete benchmark-operator pod if exist
@@ -153,14 +165,14 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def login(self):
         """
-        This method login to the cluster
+        This method logs in to the cluster
         """
         self._oc.login()
 
     @logger_time_stamp
     def tear_down_pod_after_error(self, yaml: str, pod_name: str):
         """
-        This method tear down pod in case of error
+        This method tears down pod in case of error
         @param yaml:
         @param pod_name:
         @return:
@@ -172,7 +184,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def tear_down_vm_after_error(self, yaml: str, vm_name: str):
         """
-        This method tear down vm in case of error
+        This method tears down vm in case of error
         @param yaml:
         @param vm_name:
         """
@@ -183,7 +195,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def system_metrics_collector(self, workload: str, es_fetch_min_time: int = None):
         """
-        This method run system metrics collector
+        This method runs system metrics collector
         @param workload: the workload
         @param es_fetch_min_time:
         :return:
@@ -203,10 +215,10 @@ class BenchmarkOperatorWorkloadsOperations:
             else:
                 self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check=True)
 
-    def __get_metadata(self, kind: str = None, database: str = None, status: str = None, run_artifacts_url: str = None, uuid: str = None) -> dict:
+    def __get_metadata(self, kind: str = None, database: str = None, status: str = None, run_artifacts_url: str = None, uuid: str = None, prometheus_result: dict = None) -> dict:
         """
-        This method return metadata kind and database argument are optional
-        @param kind: optional: pod, vm, or kata
+        This method returns metadata for a run, optionally filtered by runtime kind and database
+        @param kind: optionally: pod, vm, or kata
         @param database: optional:mssql, postgres or mariadb
         @param status:
         @param run_artifacts_url:
@@ -215,7 +227,8 @@ class BenchmarkOperatorWorkloadsOperations:
         date_format = '%Y_%m_%d'
         metadata = {'ocp_version': self._oc.get_ocp_server_version(),
                     'cnv_version': self._oc.get_cnv_version(),
-                    'kata_version': self._oc.get_kata_version(),
+                    'kata_version': self._oc.get_kata_operator_version(),
+                    'kata_rpm_version': self._oc.get_kata_rpm_version(node=self._pin_node1),
                     'odf_version': self._oc.get_odf_version(),
                     'runner_version': self._runner_version,
                     'version': int(self._runner_version.split('.')[-1]),
@@ -236,6 +249,8 @@ class BenchmarkOperatorWorkloadsOperations:
             metadata.update({'vm_os_version': 'fedora34'})
         if uuid:
             metadata.update({'uuid': uuid})
+        if prometheus_result:
+            metadata.update(prometheus_result)
         # for hammerdb
         if database == 'mssql':
             metadata.update({'db_version': 2019})
@@ -246,9 +261,9 @@ class BenchmarkOperatorWorkloadsOperations:
         return metadata
 
     @logger_time_stamp
-    def _update_elasticsearch_index(self, index: str, id: str, kind: str, status: str, run_artifacts_url: str, database: str = ''):
+    def _update_elasticsearch_index(self, index: str, id: str, kind: str, status: str, run_artifacts_url: str, database: str = '', prometheus_result: dict = None):
         """
-        This method update elasticsearch id
+        This method updates elasticsearch id
         :param index:
         :param id:
         :param kind:
@@ -257,11 +272,11 @@ class BenchmarkOperatorWorkloadsOperations:
         :param run_artifacts_url:
         :return:
         """
-        self.__es_operations.update_elasticsearch_index(index=index, id=id, metadata=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url))
+        self.__es_operations.update_elasticsearch_index(index=index, id=id, metadata=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url, prometheus_result=prometheus_result))
 
     def _upload_to_elasticsearch(self, index: str, kind: str, status: str, run_artifacts_url: str, database: str = '', uuid: str = ''):
         """
-        This method upload to elasticsearch
+        This method uploads data to elasticsearch
         :param index:
         :param kind:
         :param status:
@@ -273,7 +288,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _verify_elasticsearch_data_uploaded(self, index: str, uuid: str, workload: str = '', fast_check: bool = False, timeout: int = None, es_fetch_min_time: int = None):
         """
-        This method verify that elasticsearch data uploaded
+        This method verifies that elasticsearch data uploaded
         :param index:
         :param uuid:
         :param workload:
@@ -288,7 +303,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _upload_workload_to_elasticsearch(self, index: str, kind: str, status: str, result: dict = None):
         """
-        This method upload to elasticsearch
+        This method uploads to elasticsearch
         :param index:
         :param kind:
         :param status:
@@ -299,7 +314,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _verify_elasticsearch_workload_data_uploaded(self, index: str, uuid: str):
         """
-        This method verify that elasticsearch data uploaded
+        This method verifies that elasticsearch data was uploaded
         :param index:
         :param uuid:
         :return:
@@ -308,7 +323,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _create_vm_log(self, labels: list) -> str:
         """
-        This method set vm log per workload
+        This method sets vm log per workload
         :param labels: list of labels
         :return: vm_name
         """
@@ -320,7 +335,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _create_pod_log(self, label: str = '', database: str = '') -> str:
         """
-        This method create pod log per workload
+        This method creates pod log per workload
         :param label:pod label
         :param database:
         :return:
@@ -333,7 +348,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _get_run_artifacts_hierarchy(self, workload_name: str = '', is_file: bool = False):
         """
-        This method return log hierarchy
+        This method returns log hierarchy
         :param workload_name: workload name
         :param is_file: is file name
         :return:
@@ -351,7 +366,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def _create_run_artifacts(self, workload: str = '', database: str = '', labels: list = [], pod: bool = True):
         """
-        This method create pod logs of benchmark-controller-manager, system-metrics and workload pod
+        This method creates pod logs of benchmark-controller-manager, system-metrics and workload pod
         :param workload: workload name
         :param database: database name
         :param pod: False in case of vm
@@ -376,7 +391,7 @@ class BenchmarkOperatorWorkloadsOperations:
 
     def __make_run_artifacts_tarfile(self, workload: str):
         """
-        This method tar.gz log path and return the tar.gz path
+        This method compresses the log file and returns the compressed path
         :return:
         """
         tar_run_artifacts_path = f"{self._run_artifacts_path}.tar.gz"
@@ -387,7 +402,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def delete_local_artifacts(self):
         """
-        This method delete local artifacts
+        This method deletes local artifacts
         :return:
         """
         workload = self._workload_name
@@ -421,7 +436,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def start_prometheus(self):
         """
-        This method start collection of Prometheus snapshot
+        This method starts collection of Prometheus snapshot
         :return:
         """
         if self._enable_prometheus_snapshot:
@@ -435,7 +450,7 @@ class BenchmarkOperatorWorkloadsOperations:
     @logger_time_stamp
     def end_prometheus(self):
         """
-        This method retrieve the Prometheus snapshot
+        This method retrieves the Prometheus snapshot
         :return:
         """
         if self._enable_prometheus_snapshot:
@@ -447,29 +462,29 @@ class BenchmarkOperatorWorkloadsOperations:
                 raise err
 
     @logger_time_stamp
-    def odf_pvc_verification(self):
+    def odf_workload_verification(self):
         """
-        This method verified if odf or pvc is required for workload, raise error in case of missing odf
+        This method verifies whether the ODF operator is installed for ODF workloads and raises an error if it is missing.
         :return:
         """
         workload_name = self._workload.split('_')
         if workload_name[0] in self._workloads_odf_pvc:
             if not self._oc.is_odf_installed():
-                raise ODFNonInstalled()
+                raise ODFNotInstalled(workload=self._workload)
 
     @logger_time_stamp
     def verify_lso(self):
         """
-        This method Verifies that lso path exist
+        This method verifies that lso path exist
         :return:
         """
-        if not self._lso_path:
+        if not self._lso_disk_id:
             raise EmptyLSOPath()
 
     @logger_time_stamp
     def delete_all(self):
         """
-        This method delete all pod or unbound pv in namespace
+        This method deletes all pod or unbound pv in namespace
         :return:
         """
         # make undeploy benchmark controller manager if exist
@@ -478,11 +493,12 @@ class BenchmarkOperatorWorkloadsOperations:
             self._oc.delete_namespace(namespace=f"{self._workload.split('_')[2]}-db")
         if self._storage_type == 'lso':
             self._oc.delete_available_released_pv()
+            self._oc.remove_lso_path()
 
     @logger_time_stamp
     def clear_nodes_cache(self):
         """
-        This method clear nodes cache
+        This method clears nodes cache
         """
         self._oc.clear_node_caches()
 
@@ -491,10 +507,16 @@ class BenchmarkOperatorWorkloadsOperations:
         This method includes all the initialization of workload
         :return:
         """
+        # Verify that CNV operator in installed for CNV workloads
+        if '_vm' in self._workload and not self._oc.is_cnv_installed():
+            raise CNVNotInstalled(workload=self._workload)
+        # Verify that Kata operator in installed for kata workloads
+        if '_kata' in self._workload and not self._oc.is_kata_installed():
+            raise KataNotInstalled(workload=self._workload)
         self.delete_all()
         self.clear_nodes_cache()
         if self._odf_pvc:
-            self.odf_pvc_verification()
+            self.odf_workload_verification()
         if 'lso' in self._workload:
             self.verify_lso()
         # make deploy benchmark controller manager
@@ -508,6 +530,7 @@ class BenchmarkOperatorWorkloadsOperations:
         This method includes all the finalization of workload
         :return:
         """
+        self._oc.collect_events()
         if self._enable_prometheus_snapshot:
             self.end_prometheus()
         if self._endpoint_url:
