@@ -10,11 +10,11 @@ from benchmark_runner.benchmark_operator.benchmark_operator_workloads_operations
 
 class UperfPod(BenchmarkOperatorWorkloadsOperations):
     """
-    This class for uperf workload
+    This class runs uperf workload
     """
     def __init__(self):
         """
-        All inherit from WorkloadsOperations
+        All inherit from BenchmarkOperatorWorkloadsOperations
         """
         super().__init__()
         self.__name = ''
@@ -26,10 +26,11 @@ class UperfPod(BenchmarkOperatorWorkloadsOperations):
     @logger_time_stamp
     def run(self):
         """
-        This method run uperf workload
+        This method runs uperf workload
         :return:
         """
         try:
+            self._prometheus_metrics_operation.init_prometheus()
             if 'kata' in self._workload:
                 self.__kind = 'kata'
                 self.__name = self._workload.replace('kata', 'pod')
@@ -60,6 +61,10 @@ class UperfPod(BenchmarkOperatorWorkloadsOperations):
             self._oc.wait_for_ready(label='app=uperf-bench-client', workload=self.__workload_name)
             self.__status = self._oc.wait_for_pod_completed(label='app=uperf-bench-client', workload=self.__workload_name)
             self.__status = 'complete' if self.__status else 'failed'
+            # prometheus queries
+            self._prometheus_metrics_operation.finalize_prometheus()
+            metric_results = self._prometheus_metrics_operation.run_prometheus_queries()
+            prometheus_result = self._prometheus_metrics_operation.parse_prometheus_metrics(data=metric_results)
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics']:
                 self.system_metrics_collector(workload=self.__workload_name)
@@ -70,7 +75,7 @@ class UperfPod(BenchmarkOperatorWorkloadsOperations):
                 ids = self._verify_elasticsearch_data_uploaded(index=self.__es_index, uuid=self._oc.get_long_uuid(workload=self.__workload_name))
                 # update metadata
                 for id in ids:
-                    self._update_elasticsearch_index(index=self.__es_index, id=id, kind=self.__kind, status=self.__status, run_artifacts_url=run_artifacts_url)
+                    self._update_elasticsearch_index(index=self.__es_index, id=id, kind=self.__kind, status=self.__status, run_artifacts_url=run_artifacts_url, prometheus_result=prometheus_result)
             self._oc.delete_pod_sync(
                 yaml=os.path.join(f'{self._run_artifacts_path}', f'{self.__name}.yaml'),
                 pod_name=f'uperf-client')
