@@ -1,5 +1,6 @@
 
 import os
+import ast
 import time
 from enum import Enum
 from typeguard import typechecked
@@ -7,7 +8,7 @@ from typeguard import typechecked
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
 from benchmark_runner.common.oc.oc_exceptions import PodNotCreateTimeout, PodNotInitializedTimeout, PodNotReadyTimeout, \
     PodNotCompletedTimeout, PodTerminateTimeout, PodNameNotExist, LoginFailed, VMNotCreateTimeout, VMDeleteTimeout, \
-    YAMLNotExist, VMNameNotExist, VMNotInitializedTimeout, VMNotReadyTimeout, VMStateTimeout, VMNotCompletedTimeout, ExecFailed, PodFailed
+    YAMLNotExist, VMNameNotExist, VMNotInitializedTimeout, VMNotReadyTimeout, VMStateTimeout, VMNotCompletedTimeout, ExecFailed, PodFailed, DVStatusTimeout
 from benchmark_runner.common.ssh.ssh import SSH
 from benchmark_runner.main.environment_variables import environment_variables
 
@@ -33,45 +34,112 @@ class OC(SSH):
         self.__environment_variables_dict = environment_variables.environment_variables_dict
         self._run_artifacts = self.__environment_variables_dict.get('run_artifacts_path', '')
         self.__elasticsearch_url = self.__environment_variables_dict.get('elasticsearch_url', '')
+        self.__kata_csv = self.__environment_variables_dict.get('kata_csv', '')
         self.__cli = self.__environment_variables_dict.get('cli', '')
+        self.__worker_disk_prefix = self.__environment_variables_dict.get('worker_disk_prefix', '')
+        self.__worker_disk_ids = self.__environment_variables_dict.get('worker_disk_ids', '')
+        if self.__worker_disk_ids:
+            self.__worker_disk_ids = ast.literal_eval(self.__worker_disk_ids)
 
     def get_ocp_server_version(self):
         """
-        This method return ocp server version
+        This method returns ocp server version
         :return:
         """
-        return self.run(f"{self.__cli} version -ojson | jq -r '.openshiftVersion'")
+        return self.run(f"{self.__cli} get clusterversion version -o jsonpath='{{.status.desired.version}}'")
 
     def get_cnv_version(self):
         """
-        This method return cnv version
+        This method returns cnv version
         :return:
         """
-        return self.run(f"{self.__cli} get csv -n openshift-cnv $(oc get csv -n openshift-cnv --no-headers | awk '{{ print $1; }}') -ojsonpath='{{.spec.version}}'")
+        return self.run(f"{self.__cli} get csv -n openshift-cnv $({self.__cli} get csv -n openshift-cnv --no-headers | awk '{{ print $1; }}') -ojsonpath='{{.spec.version}}'")
 
     def get_odf_version(self):
         """
-        This method return odf version
+        This method returns odf version
         :return:
         """
         return self.run(f"{self.__cli} get csv -n openshift-storage -ojsonpath='{{.items[0].spec.labels.full_version}}'")
 
-    def get_kata_version(self):
+    def remove_lso_path(self):
         """
-        This method returns kata version
+        The method removes lso path on each node
+        @return:
+        """
+        self.run(fr"""{self.__cli} get nodes -l node-role.kubernetes.io/worker= -o jsonpath="{{range .items[*]}}{{.metadata.name}}{{'\\n'}}{{end}}" |  xargs -I{{}} {self.__cli} debug node/{{}} -- chroot /host sh -c "rm -rfv /mnt/local-storage/local-sc/" """)
+
+    def get_worker_disk_ids(self, node: str = None):
+        """
+        This method returns all worker disk ids or disk ids by node
+        :param node:
+        :return:
+        """
+        workers_disk_ids = []
+
+        if node and node in self.__worker_disk_ids:
+            workers_disk_ids.extend(self.__worker_disk_ids[node])
+        else:
+            for node, disk_ids in self.__worker_disk_ids.items():
+                workers_disk_ids.extend(disk_ids)
+
+        return workers_disk_ids
+
+    def get_pv_disk_ids(self):
+        """
+        This method returns list of pv disk ids
+        """
+        pv_ids = self.run(f"{self.__cli} get pv -o jsonpath={{.items[*].metadata.annotations.'storage\.openshift\.com/device-id'}}")
+        return [pv[len(self.__worker_disk_prefix):] for pv in pv_ids.split()]
+
+    def get_free_disk_id(self, node: str = None):
+        """
+        This method returns free disk per node  [workers disk ids - workers pv disk ids]
+        """
+        workers_disk_ids = self.get_worker_disk_ids(node)
+        workers_pv_disk_ids = self.get_pv_disk_ids()
+        free_disk_id = f'{self.__worker_disk_prefix}{[disk_id for disk_id in workers_disk_ids if disk_id not in workers_pv_disk_ids][0]}'
+        if free_disk_id:
+            return free_disk_id
+        else:
+            raise ValueError('Missing free disk id')
+
+    @typechecked
+    def run_debug_node(self, node: str, cmd: str):
+        """
+        This method runs command in debug node
+        :param: node
+        :param: cmd
+        :return:
+        """
+        self.run(f"{self.__cli} debug node/{node} --no-tty=true -- chroot /host sh -c '{cmd}'")
+
+    def get_kata_operator_version(self):
+        """
+        This method returns kata operator version
         :return:
         """
         return self.run(f"{self.__cli} get csv -n openshift-sandboxed-containers-operator $({self.__cli} get csv -n openshift-sandboxed-containers-operator --no-headers | awk '{{ print $1; }}') -ojsonpath='{{.spec.version}}'")
 
+    @typechecked
+    def get_kata_rpm_version(self, node: str):
+        """
+        This method returns sandboxed containers (kata) rpm version
+        @param: node
+        :return:
+        """
+        kata_rpm_version = self.run(f"{self.__cli} debug node/{node} -- chroot /host rpm -q --queryformat='%{{VERSION}}-%{{RELEASE}}' kata-containers 2>/dev/null")
+        return '.'.join(kata_rpm_version.split('.')[:3])
+
     def _get_kata_default_channel(self):
         """
-        Retrieve the default channel for Kata
+        This method retrieves the default channel for Kata
         """
         return self.run(f"{self.__cli} get packagemanifest -n openshift-marketplace sandboxed-containers-operator -ojsonpath='{{.status.defaultChannel}}'")
 
     def _get_kata_default_channel_field(self, channel_field: str):
         """
-        Retrieve a field from the packagemanifest for the default Kata channel
+        This method retrieves a field from the packagemanifest for the default Kata channel
         """
         default_channel = f'"{self._get_kata_default_channel()}"'
         command = f"{self.__cli} get packagemanifest -n openshift-marketplace sandboxed-containers-operator -ojson | jq -r '[foreach .status.channels[] as $channel ([[],[]];0;(if ($channel.name == {default_channel}) then $channel.{channel_field} else null end))] | flatten | map (select (. != null))[]'"
@@ -79,41 +147,59 @@ class OC(SSH):
 
     def _get_kata_csv(self):
         """
-        Retrieve the CSV of the sandboxed containers operator for installation"
+        This method retrieves the CSV of the sandboxed containers operator for installation"
         """
         return self._get_kata_default_channel_field("currentCSV")
 
     def _get_kata_catalog_source(self):
         """
-        Retrieve the catalog source of the sandboxed containers operator for installation"
+        This method retrieves the catalog source of the sandboxed containers operator for installation"
         """
         return self.run(f"{self.__cli} get packagemanifest -n openshift-marketplace sandboxed-containers-operator -ojsonpath='{{.status.catalogSource}}'")
 
     def _get_kata_channel(self):
         """
-        Retrieve the channel of the sandboxed containers operator for installation"
+        This method retrieves the channel of the sandboxed containers operator for installation"
         """
         return self._get_kata_default_channel_field("name")
 
     def _get_kata_namespace(self):
         """
-        Retrieve the namespace of the sandboxed containers operator for installation"
+        This method retrieves the namespace of the sandboxed containers operator for installation"
         """
         return self._get_kata_default_channel_field('currentCSVDesc.annotations."operatorframework.io/suggested-namespace"')
 
+    def set_kata_threads_pool(self, thread_pool_size: str):
+        """
+        This method sets kata thread-pool-size in every worker node
+        @param thread_pool_size:
+        @return:
+        """
+        self.run(fr"""{self.__cli} get nodes -l node-role.kubernetes.io/worker= -o jsonpath="{{range .items[*]}}{{.metadata.name}}{{'\\n'}}{{end}}" |  xargs -I{{}} {self.__cli} debug node/{{}} -- chroot /host sh -c "mkdir -p /etc/kata-containers; cp /usr/share/kata-containers/defaults/configuration.toml /etc/kata-containers/; sed -i 's/thread-pool-size=1/thread-pool-size={thread_pool_size}/' /etc/kata-containers/configuration.toml" """)
+
+    def delete_kata_threads_pool(self):
+        """
+        This method deletes kata thread-pool-size from every worker node
+        @return:
+        """
+        self.run(fr"""{self.__cli} get nodes -l node-role.kubernetes.io/worker= -o jsonpath="{{range .items[*]}}{{.metadata.name}}{{'\\n'}}{{end}}" |  xargs -I{{}} {self.__cli} debug node/{{}} -- chroot /host sh -c "rm -f /etc/kata-containers/configuration.toml" """)
+
     @typechecked
     def populate_additional_template_variables(self, env: dict):
         """
-        Populate any additional variables needed for setup templates
+        This method populates any additional variables needed for setup templates
         """
-        env['kata_csv'] = self._get_kata_csv()
-        env['kata_catalog_source'] = self._get_kata_catalog_source()
+        if self.__kata_csv:  # custom kata version
+            env['kata_csv'] = self.__kata_csv
+        else:  # latest kata version
+            env['kata_csv'] = self._get_kata_csv()
         env['kata_channel'] = self._get_kata_channel()
+        env['kata_catalog_source'] = self._get_kata_catalog_source()
         env['kata_namespace'] = self._get_kata_namespace()
 
     def is_cnv_installed(self):
         """
-        This method check if cnv operator is installed
+        This method checks if cnv operator is installed
         :return:
         """
         verify_cmd = f"{self.__cli} get csv -n openshift-cnv -ojsonpath='{{.items[0].status.phase}}'"
@@ -123,7 +209,7 @@ class OC(SSH):
 
     def is_odf_installed(self):
         """
-        This method check if odf operator is installed
+        This method checks if odf operator is installed
         :return:
         """
         verify_cmd = f"{self.__cli} get csv -n openshift-storage -ojsonpath='{{.items[0].status.phase}}'"
@@ -131,6 +217,49 @@ class OC(SSH):
             return True
         return False
 
+    def check_dv_status(self,
+                        status: str,
+                        namespace: str = environment_variables.environment_variables_dict['namespace']):
+        """
+        This method checks dv status
+        :return:
+        """
+        namespace = f'-n {namespace}' if namespace else ''
+        verify_cmd = f"{self.__cli} get dv {namespace} -ojsonpath='{{.items[].status.phase}}'"
+        if status in self.run(verify_cmd):
+            return True
+        return False
+
+    @typechecked
+    @logger_time_stamp
+    def wait_for_dv_status(self,
+                           status: str = 'Succeeded',
+                           timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
+        """
+        This method waits for methods status
+        @return: True/ False if reach to status
+        """
+        current_wait_time = 0
+        while timeout <= 0 or current_wait_time <= timeout:
+            if self.check_dv_status(status=status):
+                return True
+            # sleep for x seconds
+            time.sleep(OC.SLEEP_TIME)
+            current_wait_time += OC.SLEEP_TIME
+        raise DVStatusTimeout(status=status)
+
+    def verify_odf_installation(self, namespace: str = 'openshift-storage'):
+        """
+        This method verifies ODF installation
+        :return: True ODF passed, False failed
+        """
+        self.run(
+            f"{self.__cli} patch storagecluster ocs-storagecluster -n {namespace} --type json --patch '[{{ \"op\": \"replace\", \"path\": \"/spec/enableCephTools\", \"value\": true }}]'")
+        self.wait_for_patch(pod_name='rook-ceph-tools', label='app=rook-ceph-tools', label_uuid=False, namespace=namespace)
+        health_check = self.run(
+            f"{self.__cli} -n {namespace} rsh {self._get_pod_name(pod_name='rook-ceph-tools', namespace=namespace)} ceph health")
+        return 'HEALTH_OK' == health_check.strip()
+
     def get_odf_disk_count(self):
         """
         This method returns odf disk count
@@ -141,37 +270,38 @@ class OC(SSH):
 
     def is_kata_installed(self):
         """
-        This method check if kata operator is installed
+        This method checks if kata operator is installed
         :return:
         """
-        verify_cmd = "oc get csv -n openshift-sandboxed-containers-operator -ojsonpath='{.items[0].status.phase}'"
+        verify_cmd = f"{self.__cli} get csv -n openshift-sandboxed-containers-operator -ojsonpath='{{.items[0].status.phase}}'"
         if 'Succeeded' in self.run(verify_cmd):
             return True
         return False
 
     def get_master_nodes(self):
         """
-        This method return master nodes
+        This method returns master nodes
         :return:
         """
         return self.run(fr""" {self.__cli} get nodes -l node-role.kubernetes.io/master= -o jsonpath="{{range .items[*]}}{{.metadata.name}}{{'\n'}}{{end}}" """)
 
     def get_worker_nodes(self):
         """
-        This method return worker nodes
+        This method returns worker nodes
         :return:
         """
         return self.run(fr""" {self.__cli} get nodes -l node-role.kubernetes.io/worker= -o jsonpath="{{range .items[*]}}{{.metadata.name}}{{'\n'}}{{end}}" """)
 
     def delete_available_released_pv(self):
         """
-        This method delete available or released pv because that avoid launching new pv
+        This method deletes available or released pv because that avoid launching new pv
         """
         pv_status_list = self.run(fr"{self.__cli} get pv -ojsonpath={{..status.phase}}").split()
-        for pv_status in pv_status_list:
+        for ind, pv_status in enumerate(pv_status_list):
             if pv_status == 'Available' or pv_status == 'Released':
-                available_pv = self.run(fr"{self.__cli} get pv -ojsonpath={{.items[{pv_status_list.index(pv_status)}].metadata.name}}")
+                available_pv = self.run(fr"{self.__cli} get pv -ojsonpath={{.items[{ind}].metadata.name}}")
                 logger.info(f'Delete {pv_status} pv {available_pv}')
+                self.run(fr"{self.__cli} delete localvolume -n openshift-local-storage local-disks")
                 self.run(fr"{self.__cli} delete pv {available_pv}")
 
     def clear_node_caches(self):
@@ -182,7 +312,7 @@ class OC(SSH):
 
     def __get_short_uuid(self, workload: str):
         """
-        This method return uuid
+        This method returns uuid
         :return:
         """
         long_uuid = self.get_long_uuid(workload=workload)
@@ -192,7 +322,7 @@ class OC(SSH):
 
     def get_num_active_nodes(self):
         """
-        This method return the number of active nodes
+        This method returns the number of active nodes
         :return:
         """
         # count the number of active master/worker nodes
@@ -206,7 +336,7 @@ class OC(SSH):
     @logger_time_stamp
     def apply_security_privileged(self, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method apply security privileged for namespace
+        This method applies security privileged for namespace
         @param namespace:
         @return:
         """
@@ -217,8 +347,8 @@ class OC(SSH):
     @logger_time_stamp
     def create_async(self, yaml: str, is_check: bool = False):
         """
-        This method create yaml in async
-        :param yaml:
+        This method creates yaml in async
+        @param yaml:
         @param is_check:
         :return:
         """
@@ -231,7 +361,7 @@ class OC(SSH):
     @logger_time_stamp
     def delete_async(self, yaml: str):
         """
-        This method delete yaml in async
+        This method deletes yaml in async
         :param yaml:
         :return:
         """
@@ -244,7 +374,7 @@ class OC(SSH):
     def _get_pod_name(self, pod_name: str,
                       namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method return pod name if exist or raise error
+        This method returns pod name if exist or raise error
         :param pod_name:
         :param namespace:
         :return:
@@ -258,7 +388,7 @@ class OC(SSH):
     def pod_exists(self, pod_name: str,
                    namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method return True if exist or False if not
+        This method returns True if exist or False if not
         :param pod_name:
         :param namespace:
         :return:
@@ -273,7 +403,7 @@ class OC(SSH):
     def pod_label_exists(self, label_name: str,
                          namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method return True if pod exist or not by label
+        This method returns True if pod exist or not by label
         :param label_name:
         :param namespace:
         :return:
@@ -288,7 +418,7 @@ class OC(SSH):
     @typechecked()
     def get_long_uuid(self, workload: str):
         """
-        This method return uuid
+        This method returns uuid
         :return:
         """
         long_uuid = self.run(
@@ -311,7 +441,7 @@ class OC(SSH):
 
     def get_prom_token(self):
         """
-        This method return prometheus token
+        This method returns prometheus token
         :return:
         """
         # OCP 4.10 and below
@@ -321,10 +451,19 @@ class OC(SSH):
             prom_token = self.run(f"{self.__cli} sa new-token -n openshift-monitoring prometheus-k8s 2>/dev/null")
         return prom_token
 
+    def collect_events(self):
+        """
+        This method collects event log
+        :return: output_filename
+        """
+        output_filename = os.path.join(self._run_artifacts, f'events.log')
+        self.run(f"{self.__cli} get events -A > '{output_filename}' ")
+        return output_filename
+
     @logger_time_stamp
     def login(self):
         """
-        This method login to the cluster
+        This method logs in to the cluster
         :return:
         """
         try:
@@ -338,7 +477,7 @@ class OC(SSH):
     @logger_time_stamp
     def get_pod(self, label: str, database: str = '', namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method get pods according to label
+        This method gets pods according to label
         :param label:
         :param database:
         :param namespace:
@@ -354,27 +493,42 @@ class OC(SSH):
 
     @typechecked
     @logger_time_stamp
-    def save_pod_log(self, pod_name: str, database: str = ''):
+    def save_pod_log(self, pod_name: str, database: str = '', log_type: str = ''):
         """
-        This method save pod log in log_path
+        This method saves pod log in log_path
         :param pod_name: pod name with uuid
         :param database: database
+        :param log_type: log type extension
         :return: output_filename
         """
-        output_filename = os.path.join(self._run_artifacts, pod_name)
+        if log_type:
+            output_filename = os.path.join(self._run_artifacts, f'{pod_name}{log_type}')
+        else:
+            output_filename = os.path.join(self._run_artifacts, pod_name)
         if database:
-            self.run(f"{self.__cli} logs -n '{database}-db' {pod_name} > {output_filename} ")
+            self.run(f"{self.__cli} logs -n '{database}-db' {pod_name} > '{output_filename}' ")
         # manager logs of benchmark-controller-manager
         elif 'benchmark-controller-manager' in pod_name:
-            self.run(f"{self.__cli} logs -n {environment_variables.environment_variables_dict['namespace']} {pod_name} manager > {output_filename} ")
+            self.run(f"{self.__cli} logs -n {environment_variables.environment_variables_dict['namespace']} {pod_name} manager > '{output_filename}' ")
         else:
-            self.run(f"{self.__cli} logs -n {environment_variables.environment_variables_dict['namespace']} {pod_name} > {output_filename} ")
+            self.run(f"{self.__cli} logs -n {environment_variables.environment_variables_dict['namespace']} {pod_name} > '{output_filename}' ")
+        return output_filename
+
+    def describe_pod(self, pod_name: str, namespace: str = ''):
+        """
+        This method describes pod into log
+        :param pod_name: pod name with uuid
+        :param namespace: namespace
+        :return: output_filename
+        """
+        output_filename = os.path.join(self._run_artifacts, f'describe-{pod_name}')
+        self.run(f"{self.__cli} describe pod -n {namespace} {pod_name} > '{output_filename}' ")
         return output_filename
 
     @logger_time_stamp
     def get_pods(self):
         """
-        This method get pods
+        This method retrieves information on benchmark-runner pods in oc get pod format
         :return:
         """
         return self.run(f'{self.__cli} get pods', is_check=True)
@@ -385,7 +539,7 @@ class OC(SSH):
                             namespace: str = environment_variables.environment_variables_dict['namespace'],
                             timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method is wait till pod name is creating or throw exception after timeout
+        This method waits till pod name is creating or throw exception after timeout
         :param namespace:
         :param pod_name:
         :param timeout:
@@ -394,10 +548,12 @@ class OC(SSH):
         current_wait_time = 0
         while timeout <= 0 or current_wait_time <= timeout:
             if self.pod_exists(pod_name=pod_name, namespace=namespace):
+                self.describe_pod(pod_name=pod_name, namespace=namespace)
                 return True
             # sleep for x seconds
             time.sleep(OC.SLEEP_TIME)
             current_wait_time += OC.SLEEP_TIME
+        self.describe_pod(pod_name=pod_name, namespace=namespace)
         raise PodNotCreateTimeout(pod_name)
 
     @typechecked
@@ -406,7 +562,7 @@ class OC(SSH):
                                namespace: str = environment_variables.environment_variables_dict['namespace'],
                                timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method is wait till pod name is terminating or throw exception after timeout
+        This method waits till pod name is terminating or throw exception after timeout
         :param namespace:
         :param pod_name:
         :param timeout:
@@ -427,7 +583,7 @@ class OC(SSH):
                         namespace: str = environment_variables.environment_variables_dict['namespace'],
                         timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method create pod yaml in async
+        This method creates pod yaml in async
         :param namespace:
         :param timeout:
         :param pod_name:
@@ -443,7 +599,7 @@ class OC(SSH):
                         namespace: str = environment_variables.environment_variables_dict['namespace'],
                         timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method delete pod yaml in async, only if exist and return false if not exist
+        This method deletes pod yaml in async, only if exist and return false if not exist
         :param namespace:
         :param timeout:
         :param pod_name:
@@ -459,7 +615,7 @@ class OC(SSH):
     @typechecked
     def delete_namespace(self, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method delete namespace
+        This method deletes namespace
         :param namespace:
         :return:
         """
@@ -471,7 +627,7 @@ class OC(SSH):
                              namespace: str = environment_variables.environment_variables_dict['namespace'],
                              timeout: int = SHORT_TIMEOUT):
         """
-        This method wait to pod to be initialized
+        This method waits to pod to be initialized
         :param namespace:
         :param label:
         :param status:
@@ -498,6 +654,27 @@ class OC(SSH):
             else:
                 raise PodNotInitializedTimeout(workload=workload)
 
+    @typechecked
+    @logger_time_stamp
+    def wait_for_patch(self, pod_name: str, label: str, label_uuid: bool, namespace: str, timeout: int = SHORT_TIMEOUT):
+        """
+        This method waits for patch, needs to wait that pod is created and then wait for ready
+        @param pod_name:
+        @param label:
+        @param label_uuid:
+        @param namespace:
+        @param timeout:
+        @return:
+        """
+        current_wait_time = 0
+        while timeout <= 0 or current_wait_time <= timeout:
+            if self._get_pod_name(pod_name=pod_name, namespace=namespace) and self.wait_for_ready(label=label, label_uuid=label_uuid, namespace=namespace):
+                return True
+            # sleep for x seconds
+            time.sleep(OC.SLEEP_TIME)
+            current_wait_time += OC.SLEEP_TIME
+        raise PodNotReadyTimeout(label)
+
     @typechecked
     @logger_time_stamp
     def wait_for_ready(self, label: str, run_type: str = 'pod', workload: str = '', status: str = 'ready', label_uuid: bool = True,
@@ -535,7 +712,7 @@ class OC(SSH):
     @typechecked
     def exec(self, command: str, pod_name: str, namespace: str = environment_variables.environment_variables_dict['namespace'], container: str = ''):
         """
-        oc exec a command and return the answer
+        This method executes a command within a specified pod and optional container and returns the output
         :param command:
         :param pod_name:
         :param namespace:
@@ -554,7 +731,7 @@ class OC(SSH):
                            namespace: str = environment_variables.environment_variables_dict['namespace'],
                            timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        Delete a pod based on name and namespace only
+        This method deletes a pod based on name and namespace only
         :param pod_name:
         :param namespace:
         :param timeout:
@@ -573,7 +750,7 @@ class OC(SSH):
                            namespace: str = environment_variables.environment_variables_dict['namespace'],
                            timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        Wait for a pod to be ready and running
+        This method waits for a pod to be ready and running
         :param pod_name:
         :param namespace:
         :param timeout:
@@ -598,7 +775,7 @@ class OC(SSH):
                                namespace: str = environment_variables.environment_variables_dict['namespace'],
                                timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method wait to pod to be completed
+        This method waits for a pod to be completed
         :param workload:
         :param label:
         :param label_uuid: need to get uuid from label (benchmark-operator)
@@ -630,6 +807,17 @@ class OC(SSH):
         except Exception as err:
             raise PodNotCompletedTimeout(workload=workload)
 
+    def describe_vmi(self, vm_name: str, namespace: str):
+        """
+        This method describes vmi into log
+        :param vm_name: vm name with uuid
+        :param namespace: namespace
+        :return: output_filename
+        """
+        output_filename = os.path.join(self._run_artifacts, f'describe-{vm_name}')
+        self.run(f"{self.__cli} describe vmi -n {namespace} {vm_name} > '{output_filename}' ")
+        return output_filename
+
     @typechecked
     def _get_vm_name(self, vm_name: str, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
@@ -647,7 +835,7 @@ class OC(SSH):
     @typechecked
     def vm_exists(self, vm_name: str, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method return True or False if vm name exist
+        This method returns True or False if vm name exist
         :param vm_name:
         :param namespace:
         :return: True or False
@@ -663,7 +851,7 @@ class OC(SSH):
     @logger_time_stamp
     def get_vm(self, label: str = '', namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method get vm according to label
+        This method gets vm according to label
         :param label:
         :param namespace:
         :return:
@@ -678,7 +866,7 @@ class OC(SSH):
     @logger_time_stamp
     def __verify_vm_log_complete(self, vm_name: str, timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method verify that vm log is complete
+        This method verifies that vm log is complete
         :param vm_name: vm name with uuid
         :return:
         """
@@ -697,7 +885,7 @@ class OC(SSH):
     @logger_time_stamp
     def get_exposed_vm_port(self, vm_name: str, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        The method get exposed vm port
+        The method gets exposed vm port
         @param vm_name:
         @return:
         """
@@ -719,7 +907,7 @@ class OC(SSH):
                            namespace: str = environment_variables.environment_variables_dict['namespace'],
                            timeout: int = SHORT_TIMEOUT):
         """
-        This method wait for VM to reach the specified status
+        This method waits for VM to reach the specified status
         :param vm_name:
         :param status: Stopped, Starting, Running
         :param namespace:
@@ -762,7 +950,7 @@ class OC(SSH):
     @logger_time_stamp
     def get_vm_node(self, vm_name: str, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method get vm node
+        This method gets vm node
         :param vm_name:
         :param namespace:
         :return:
@@ -776,7 +964,7 @@ class OC(SSH):
                            namespace: str = environment_variables.environment_variables_dict['namespace'],
                            timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method is wait till vm name is creating or throw exception after timeout
+        This method waits till vm name is creating or throw exception after timeout
         :param vm_name:
         :param namespace:
         :param timeout:
@@ -785,10 +973,12 @@ class OC(SSH):
         current_wait_time = 0
         while timeout <= 0 or current_wait_time <= timeout:
             if self.vm_exists(vm_name=vm_name, namespace=namespace):
+                self.describe_vmi(vm_name=vm_name, namespace=namespace)
                 return True
             # sleep for x seconds
             time.sleep(OC.SLEEP_TIME)
             current_wait_time += OC.SLEEP_TIME
+        self.describe_vmi(vm_name=vm_name, namespace=namespace)
         raise VMNotCreateTimeout(vm_name)
 
     @typechecked
@@ -797,7 +987,7 @@ class OC(SSH):
                        namespace: str = environment_variables.environment_variables_dict['namespace'],
                        timeout: int = int(environment_variables.environment_variables_dict['timeout'])):
         """
-        This method create vm synchronously
+        This method creates vm synchronously
         :param timeout:
         :param vm_name:
         :param yaml:
@@ -828,7 +1018,7 @@ class OC(SSH):
     @logger_time_stamp
     def delete_all_vms(self, namespace: str = environment_variables.environment_variables_dict['namespace']):
         """
-        This method delete all vms
+        This method deletes all vms
         :return:
         """
         namespace = f'-n {namespace}' if namespace else ''
@@ -909,7 +1099,7 @@ class OC(SSH):
     @logger_time_stamp
     def extract_vm_results(self, vm_name: str = '', start_stamp: str = '', end_stamp: str = '', output_filename: str = ''):
         """
-        This method extract vm results from vm output log
+        This method extracts vm results from vm output log
         :param vm_name:
         :param start_stamp: start of run stamp
         :param end_stamp: end of run stamp
@@ -922,15 +1112,15 @@ class OC(SSH):
             output_filename = os.path.join(self._run_artifacts, vm_name)
         results_list = []
         with open(output_filename) as infile:
-            copy = False
+            # VM logs results start
+            start = False
             for line in infile:
-                if start_stamp in line:
-                    copy = True
+                if start_stamp in line or 'podman' in line and not start:
+                    start = True
                     continue
                 elif end_stamp in line:
-                    copy = False
-                    continue
-                elif copy:
+                    return results_list
+                elif start:
                     # Filter 'cloud-init' and CSV lines only
                     if 'cloud-init' in line and ',' in line:
                         if vm_name in line:
