@@ -8,17 +8,18 @@ from urllib3.exceptions import InsecureRequestWarning
 from urllib3 import disable_warnings
 disable_warnings(InsecureRequestWarning)
 
+from benchmark_runner.main.environment_variables import environment_variables
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
-from benchmark_runner.workloads.workloads_operations import WorkloadsOperations
 
 
-class PrometheusMetricsOperation(WorkloadsOperations):
+class PrometheusMetricsOperation:
     """
-    This class Run prometheus queries
+    This class contains methods for running prometheus queries, parsing the results into dictionary nad uploading it into elasticsearch
     """
 
     def __init__(self):
         super().__init__()
+        self._environment_variables_dict = environment_variables.environment_variables_dict
         self.__current_dir = os.path.dirname(os.path.abspath(__file__))
         self.__queries_file = os.path.join(self.__current_dir, 'metrics-default.yaml')
         self.__authorization = {'Authorization': self.__get_prometheus_token()}
@@ -26,6 +27,9 @@ class PrometheusMetricsOperation(WorkloadsOperations):
         self.__metrics_start_time = None
         self.__metrics_end_time = None
         self.__metric_results = {}
+        self._prometheus_snap_interval = self._environment_variables_dict.get('prometheus_snap_interval', '')
+        self._es_host = self._environment_variables_dict.get('elasticsearch', '')
+        self._es_port = self._environment_variables_dict.get('elasticsearch_port', '')
 
     @staticmethod
     def __get_prometheus_default_url():
@@ -160,4 +164,33 @@ class PrometheusMetricsOperation(WorkloadsOperations):
                 print(query, results)
         else:
             raise Exception('Missing ElasticSearch data')
-        
+
+    @staticmethod
+    @logger_time_stamp
+    def parse_prometheus_metrics(data: dict):
+        """
+        This method parses prometheus metrics and returns summary result
+        based on queries: /benchmark_runner/common/prometheus/metrics-default.yaml
+        @return:
+        """
+        result_dict = {}
+        for query, data_list in data.items():
+            if 'containerCPU-benchmark-runner' in query:
+                suffix = 'CPU'
+            elif 'containerMemory-benchmark-runner' in query:
+                suffix = 'Memory'
+            else:
+                suffix = None
+            total = 0
+            max = 0
+            for item in data_list:
+                for val in item['values']:
+                    if float(val[1]) > max:
+                        max = round(float(val[1]), 3)
+                total = total + max
+                if not suffix:
+                    result_dict[f'{query}'] = max
+                else:
+                    result_dict[f"{item['metric']['node']}_{suffix}"] = max
+                    result_dict[f'total_{suffix}'] = total
+        return result_dict
